/*  320x240,  ILI9341 */

#include"Port.h"
#include "Font8x8.inc"

#define LCD_Reg_COM (*(volatile u16*)0x60000000)
#define LCD_Reg_DAT (*(volatile u16*)0x60020000)
#define LCD_Reg_DAT32 (*(volatile u32*)0x60020000)

#define LCD_RST(x) GPIO_##x##Bits(GPIOB,GPIO_Pin_11)
#define LCD_BL(x)  GPIO_##x##Bits(GPIOD,GPIO_Pin_12)

#define GAMMA

u16 LCD_Back=0x0000; /* BackGround Color */
u16 LCD_Key =0x0000; /* Key Color        */

void delay_ms(volatile u32 d)
{
 d*=63000;
 while(d--);
}

void LCD_Reset(void)
{
 LCD_RST(Reset);
 delay_ms(100);
 LCD_RST(Set);
 delay_ms(10);
}

void LCD_LED(u8 led)
{
 if(led)LCD_BL(Set);
 else LCD_BL(Reset);
}

void LCD_Init(void)
{

 LCD_Reg_COM=0xCB;
 LCD_Reg_DAT=0x39; 
 LCD_Reg_DAT=0x2C; 
 LCD_Reg_DAT=0x00; 
 LCD_Reg_DAT=0x34; 
 LCD_Reg_DAT=0x02; 

 LCD_Reg_COM=0xCF;  
 LCD_Reg_DAT=0x00; 
 LCD_Reg_DAT=0xC1; 
 LCD_Reg_DAT=0x30; 

 LCD_Reg_COM=0xE8;  
 LCD_Reg_DAT=0x85; 
 LCD_Reg_DAT=0x00; 
 LCD_Reg_DAT=0x78; 

 LCD_Reg_COM=0xEA;  
 LCD_Reg_DAT=0x00; 
 LCD_Reg_DAT=0x00; 
 
 LCD_Reg_COM=0xED;  
 LCD_Reg_DAT=0x64; 
 LCD_Reg_DAT=0x03; 
 LCD_Reg_DAT=0x12; 
 LCD_Reg_DAT=0x81; 

 LCD_Reg_COM=0xF7;  
 LCD_Reg_DAT=0x20; 
  
 LCD_Reg_COM=0xC0;    //Power control 
 LCD_Reg_DAT=0x23;    //VRH[5:0] 
 
 LCD_Reg_COM=0xC1;    //Power control 
 LCD_Reg_DAT=0x10;    //SAP[2:0];BT[3:0] 

 LCD_Reg_COM=0xC5;    //VCM control 
 LCD_Reg_DAT=0x3E;    //Contrast
 LCD_Reg_DAT=0x28; 
 
 LCD_Reg_COM=0xC7;    //VCM control2 
 LCD_Reg_DAT=0x86;    //--
 
 LCD_Reg_COM=0x36;    //Memory Access Control 
 LCD_Reg_DAT=0x48;   

 LCD_Reg_COM=0x3A;    
 LCD_Reg_DAT=0x55; 

 LCD_Reg_COM=0xB1;    
 LCD_Reg_DAT=0x00;  
 LCD_Reg_DAT=0x18; 
 
 LCD_Reg_COM=0xB6;    //Display Function Control 
 LCD_Reg_DAT=0x08; 
 LCD_Reg_DAT=0x82;
 LCD_Reg_DAT=0x27;  

#ifdef GAMMA
 LCD_Reg_COM=0xF2;    //3Gamma Function Disable 
 LCD_Reg_DAT=0x00; 
 
 LCD_Reg_COM=0x26;    //Gamma curve selected 
 LCD_Reg_DAT=0x01; 

 LCD_Reg_COM=0xE0;    //Set Gamma 
 LCD_Reg_DAT=0x0F; 
 LCD_Reg_DAT=0x31; 
 LCD_Reg_DAT=0x2B; 
 LCD_Reg_DAT=0x0C; 
 LCD_Reg_DAT=0x0E; 
 LCD_Reg_DAT=0x08; 
 LCD_Reg_DAT=0x4E; 
 LCD_Reg_DAT=0xF1; 
 LCD_Reg_DAT=0x37; 
 LCD_Reg_DAT=0x07; 
 LCD_Reg_DAT=0x10; 
 LCD_Reg_DAT=0x03; 
 LCD_Reg_DAT=0x0E; 
 LCD_Reg_DAT=0x09; 
 LCD_Reg_DAT=0x00; 

 LCD_Reg_COM=0xE1;    //Set Gamma 
 LCD_Reg_DAT=0x00; 
 LCD_Reg_DAT=0x0E; 
 LCD_Reg_DAT=0x14; 
 LCD_Reg_DAT=0x03; 
 LCD_Reg_DAT=0x11; 
 LCD_Reg_DAT=0x07; 
 LCD_Reg_DAT=0x31; 
 LCD_Reg_DAT=0xC1; 
 LCD_Reg_DAT=0x48; 
 LCD_Reg_DAT=0x08; 
 LCD_Reg_DAT=0x0F; 
 LCD_Reg_DAT=0x0C; 
 LCD_Reg_DAT=0x31; 
 LCD_Reg_DAT=0x36; 
 LCD_Reg_DAT=0x0F; 
#endif

 LCD_Reg_COM=0x11;    //Exit Sleep 
 delay_ms(120); 
				
 LCD_Reg_COM=0x29;    //Display on 
 LCD_Reg_COM=0x2C; 
 
 //Additional 
 LCD_Reg_COM=0xB1;    //Frame Rate 119 Hz
 LCD_Reg_DAT=0x00;
 LCD_Reg_DAT=0x10;
 
 //LCD_Reg_COM=0x36;    //Exchange: X[0..239] & Y[0..319] => X[0..319] & Y[0..239], RGB => BGR
 //LCD_Reg_DAT=(1<<5)|(1<<3);
}

void LCD_Position(u16 x1,u16 x2,u16 y1,u16 y2)
{
 LCD_Reg_COM=0x2A;
 LCD_Reg_DAT=x1>>8;
 LCD_Reg_DAT=x1;
 LCD_Reg_DAT=x2>>8;
 LCD_Reg_DAT=x2;

 LCD_Reg_COM=0x2B;
 LCD_Reg_DAT=y1>>8;
 LCD_Reg_DAT=y1;
 LCD_Reg_DAT=y2>>8;
 LCD_Reg_DAT=y2;
 
 LCD_Reg_COM=0x2C;
}

void LCD_Clear(u32 color)
{
 register u32 i=(320*240)>>1;
 color|=(color<<16);
 LCD_Position(0,239,0,319);
 while(i--)LCD_Reg_DAT32=color;
}

void LCD_Quad(u16 x,u16 y,u16 s,u16 c)
{
 register u32 i;
 LCD_Position(x,s+x-1,y,s+y-1);
 for(i=0;i<s*s;i++)LCD_Reg_DAT=c;
}

void LCD_Char(u16 x,u8 y,u8 c,u16 k)
{
 register u32 i;
 LCD_Position(x<<3,(x<<3)|7,y<<3,(y<<3)|7);
 for(i=0;i<64;i++)
 {
  if(((Font8x8[(c<<3)+(i>>3)]>>(7-(i&7)))&1))LCD_Reg_DAT=k;
  else LCD_Reg_DAT=LCD_Back;
 }
}

void LCD_String(u16 x,u8 y,char* s,u16 k)
{
 register u32 i=0;
 while(s[i])
 {
  LCD_Char(x+(i%30),y+(i/30),s[i],k);
  i++;
 }
}

void LCD_Number(u16 x,u8 y,u32 n,u16 k,s8 l)
{
 if(l>0)LCD_Char(x+l-1,y, n          %10+'0',k);
 l--;
 if(l>0)LCD_Char(x+l-1,y,(n/      10)%10+'0',k);
 l--;
 if(l>0)LCD_Char(x+l-1,y,(n/     100)%10+'0',k);
 l--;
 if(l>0)LCD_Char(x+l-1,y,(n/    1000)%10+'0',k);
 l--;
 if(l>0)LCD_Char(x+l-1,y,(n/   10000)%10+'0',k);
 l--;
 if(l>0)LCD_Char(x+l-1,y,(n/  100000)%10+'0',k);
 l--;
 if(l>0)LCD_Char(x+l-1,y,(n/ 1000000)%10+'0',k);
 l--;
 if(l>0)LCD_Char(x+l-1,y,(n/10000000)%10+'0',k);
}

void LCD_HEX4(u16 x,u8 y,u16 n,u16 k)
{
 LCD_Char(x,y,HEX_String[n&0xF],k);
}

void LCD_HEX8(u16 x,u8 y,u16 n,u16 k)
{
 LCD_HEX4(x  ,y,n>>4,k);
 LCD_HEX4(x+1,y,n,k);
}

void LCD_HEX16(u16 x,u8 y,u16 n,u16 k)
{
 LCD_HEX8(x  ,y,n>>8,k);
 LCD_HEX8(x+2,y,n,k);
}

void LCD_Sprite(register u16 x,register u16 y,register u16 w,register u16 h,register const u8* s)
{
 register u32 i=(w*h)>>1;
 register u32 *S=(u32*)s;
 LCD_Position(x,x+w-1,y,y+h-1);
 while(i--)LCD_Reg_DAT32=S++[0];
}
